"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Container = void 0;
const rustic_1 = require("rustic");
const HashMap_1 = require("../internal/HashMap");
const defaultRenderHashMap = {
    add(_ignoredKey, ignoredValue) {
        return this;
    },
    get(key) {
        return key;
    },
};
class Container {
    constructor(renderConstructor, source) {
        this.cache = new HashMap_1.HashMap();
        this.renderConstructor = renderConstructor;
        this.source = source;
    }
    async fetch(key) {
        let renderer = this.cache.get(key);
        if (renderer != null) {
            return (0, rustic_1.Ok)(renderer);
        }
        const result = await this.source.fetch(key);
        if ((0, rustic_1.isErr)(result)) {
            console.warn(`I18N Render Lookup Failed for: ${key}`, result.data);
            renderer = this.renderConstructor.make(key, defaultRenderHashMap);
        }
        else {
            renderer = this.renderConstructor.make(key, result.data);
        }
        this.cache.add(key, renderer);
        return (0, rustic_1.Ok)(renderer);
    }
}
exports.Container = Container;
//# sourceMappingURL=container.js.map