"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
const rustic_1 = require("rustic");
const fail_1 = require("../internal/fail");
const future_1 = require("../internal/future");
const HashMap_1 = require("../internal/HashMap");
const gettext_parser_1 = require("gettext-parser");
function getDefaultTranslationsContext(po) {
    return po.translations[""];
}
function poFileParse(contents) {
    const fn = () => gettext_parser_1.po.parse(contents);
    const errorHandler = (error) => (error instanceof SyntaxError)
        ? new fail_1.FailParse(error.message)
        : null;
    return (0, fail_1.tryIntoResult)(fn, errorHandler);
}
function fail(error) {
    return future_1.Future.error(error).inner;
}
function parse(identifier, contents) {
    const parsed = poFileParse(contents);
    if ((0, rustic_1.isErr)(parsed)) {
        return fail(parsed.data);
    }
    const context = getDefaultTranslationsContext(parsed.data);
    if (context == null) {
        const msg = `PO FILE :: INVALID :: ${identifier} :: Missing default translation context`;
        return fail(new fail_1.FailParse(msg));
    }
    const hash = new HashMap_1.HashMap();
    Object.keys(context).forEach((key) => {
        const message = context[key];
        const msgstr = message.msgstr[0];
        if (msgstr == null) {
            const warning = `PO FILE :: INVALID :: ${identifier} :: ${key} as an empty \`msgstr\` entry`;
            console.warn(warning);
        }
        else {
            hash.add(message.msgid, msgstr);
        }
    });
    return future_1.Future.resolve(hash).inner;
}
exports.parse = parse;
//# sourceMappingURL=file-po.js.map