"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HandlebarsConstructor = exports.HandlebarsRenderer = void 0;
const handlebars_1 = __importDefault(require("handlebars"));
const rustic_1 = require("rustic");
const HBS_GLOBAL = handlebars_1.default.create();
function makeHelper(i18n, cache) {
    return function i18nHelper(aString) {
        let template = cache.get(aString);
        if (template == null) {
            const msgstr = (0, rustic_1.equip)(i18n.get(aString)).unwrapOr(aString);
            template = HBS_GLOBAL.compile(msgstr);
            if (template == null) {
                throw new Error(`PANIC: recieved a null value when attempting to compile: ${msgstr}`);
            }
            cache.add(aString, template);
        }
        return template(this);
    };
}
class HandlebarsRenderer {
    constructor(lang, i18n, cache) {
        this.env = handlebars_1.default.create();
        this.env.registerHelper('i18n', makeHelper(i18n, cache));
        this.languageCode = lang;
    }
    getName() {
        return this.languageCode;
    }
    async render(template, context) {
        return (0, rustic_1.catchResult)(() => this.env.compile(template)(context));
    }
}
exports.HandlebarsRenderer = HandlebarsRenderer;
class HandlebarsConstructor {
    constructor(cacheConstructor) {
        this.cacheConstructor = cacheConstructor;
    }
    make(lang, store) {
        return new HandlebarsRenderer(lang, store, this.cacheConstructor());
    }
}
exports.HandlebarsConstructor = HandlebarsConstructor;
//# sourceMappingURL=handlebars.js.map