"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeFS = void 0;
const fs_1 = require("fs");
const promises_1 = __importDefault(require("fs/promises"));
const rustic_1 = require("rustic");
const errorToString = (e) => {
    if (typeof e === 'string') {
        return e;
    }
    else if (e instanceof Error) {
        return e.message;
    }
    else {
        return JSON.stringify(e);
    }
};
exports.NodeFS = {
    async canRead(path) {
        try {
            await promises_1.default.access(path, fs_1.constants.R_OK);
            return (0, rustic_1.Ok)(true);
        }
        catch (_ignoreError) {
            return (0, rustic_1.Ok)(false);
        }
    },
    async readFile(path) {
        try {
            return (0, rustic_1.Ok)(await promises_1.default.readFile(path, { encoding: 'utf8' }));
        }
        catch (e) {
            return (0, rustic_1.Err)(errorToString(e));
        }
    },
    async stat(path) {
        try {
            return (0, rustic_1.Ok)(await promises_1.default.stat(path));
        }
        catch (e) {
            return (0, rustic_1.Err)(errorToString(e));
        }
    },
};
//# sourceMappingURL=file-system.js.map