"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Future = void 0;
const rustic_1 = require("rustic");
class Future {
    constructor(promise) {
        this.promise = promise;
    }
    get inner() {
        return this.promise;
    }
    then(onResolved, _onRejected) {
        this.promise.then(onResolved);
    }
    andThen(transform) {
        const handler = (result) => (0, rustic_1.isErr)(result) ? Promise.resolve(result) : transform(result.data);
        return new Future(this.promise.then(handler));
    }
    andThen2(x, transform) {
        const promise = Promise.all([this.promise, x]).then(([a, b]) => {
            if ((0, rustic_1.isErr)(a)) {
                return a;
            }
            if ((0, rustic_1.isErr)(b)) {
                return b;
            }
            return transform(a.data, b.data);
        });
        return new Future(promise);
    }
    andThen3(x, y, transform) {
        const promise = Promise.all([this.promise, x, y]).then(([a, b, c]) => {
            if ((0, rustic_1.isErr)(a)) {
                return a;
            }
            if ((0, rustic_1.isErr)(b)) {
                return b;
            }
            if ((0, rustic_1.isErr)(c)) {
                return c;
            }
            return transform(a.data, b.data, c.data);
        });
        return new Future(promise);
    }
    andThenFuture(transform) {
        const handler = (result) => (0, rustic_1.isErr)(result) ? Promise.resolve(result) : transform(result.data).inner;
        return Future.equip(this.promise.then(handler));
    }
    andThenResult(transform) {
        const handler = (result) => Promise.resolve((0, rustic_1.isErr)(result) ? result : transform(result.data));
        return new Future(this.promise.then(handler));
    }
    map(transform) {
        const handler = (out) => {
            return (0, rustic_1.equip)(out).map((out) => transform(out)).inner;
        };
        return new Future(this.promise.then(handler));
    }
    map2(x, transform) {
        const promise = Promise.all([this.promise, x]).then(([a, b]) => {
            if ((0, rustic_1.isErr)(a)) {
                return a;
            }
            if ((0, rustic_1.isErr)(b)) {
                return b;
            }
            return (0, rustic_1.Ok)(transform(a.data, b.data));
        });
        return new Future(promise);
    }
    mapErr(transform) {
        const handler = (result) => (0, rustic_1.isErr)(result) ? (0, rustic_1.Err)(transform(result.data)) : (0, rustic_1.Ok)(result.data);
        return new Future(this.promise.then(handler));
    }
    static equip(promise) {
        return new Future(promise);
    }
    static error(error) {
        return Future.fromResult((0, rustic_1.Err)(error));
    }
    static fromResult(result) {
        return new Future(Promise.resolve(result));
    }
    static resolve(value) {
        const result = (0, rustic_1.Ok)(value);
        return new Future(Promise.resolve(result));
    }
}
exports.Future = Future;
//# sourceMappingURL=future.js.map