#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const backtrace_service_1 = require("backtrace-service");
const rustic_1 = require("rustic");
const from_backtrace_service_1 = require("./app/from-backtrace-service");
const backtraceService_1 = require("./configuration/backtraceService");
const integrationRoutes_1 = require("./integrations/integrationRoutes");
const templateRoutes_1 = require("./integrations/templateRoutes");
const logger = backtrace_service_1.Logger.getLogger();
function initializeRoutes(translation) {
    const integrationRoutes = new integrationRoutes_1.IntegrationRoutes(logger, backtraceService_1.service.configuration.smtp.from, translation, backtraceService_1.service.identityManager, backtraceService_1.service.configuration.whitelistedIpAddresses || []);
    backtraceService_1.service.app.use(backtraceService_1.service.apiPrefix, integrationRoutes.configure());
    const templateRoutes = new templateRoutes_1.TemplateRoutes(logger, translation, backtraceService_1.service.identityManager);
    const templatePrefix = `${backtraceService_1.service.apiPrefix}/template`;
    backtraceService_1.service.app.use(templatePrefix, templateRoutes.configure());
}
async function startup() {
    const translation = await (0, from_backtrace_service_1.makeAppFromService)(backtraceService_1.Service, null);
    if ((0, rustic_1.isErr)(translation)) {
        throw new Error(`App load failed: ${translation.data}`);
    }
    initializeRoutes(translation.data);
    backtraceService_1.service.start();
}
startup();
//# sourceMappingURL=main.js.map