"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MjmlFileLoader = exports.MjmlTemplateLoader = void 0;
const mjml_1 = __importDefault(require("mjml"));
const rustic_1 = require("rustic");
const result_1 = require("../internal/result");
const future_1 = require("../internal/future");
const source_1 = require("../source");
const template_1 = require("../template");
const FrontMatter = __importStar(require("./front-matter"));
const mjmlConfig = {
    keepComments: false,
};
function mjmlRender(input, _config) {
    try {
        const results = (0, mjml_1.default)(input, mjmlConfig);
        return (0, rustic_1.Ok)(results.html);
    }
    catch (e) {
        if (e instanceof Error) {
            return (0, rustic_1.Err)(e.message);
        }
        return (0, rustic_1.Err)(String(e));
    }
}
function mjmlValidate(input) {
    try {
        (0, mjml_1.default)(input, { validationLevel: 'strict' });
        return (0, rustic_1.Ok)(input);
    }
    catch (e) {
        return (0, rustic_1.Err)(JSON.stringify(e));
    }
}
class Template {
    constructor(config) {
        this.config = config;
    }
    getContent(i18n, context) {
        return future_1.Future.equip(i18n.render(this.config.content, context))
            .andThenResult((i18nContent) => mjmlRender(i18nContent, mjmlConfig))
            .inner;
    }
    getName() {
        return this.config.name;
    }
    getSubject(i18n, context) {
        return i18n.render(this.config.subject, context);
    }
}
class TemplateParser {
    parse(source) {
        const r1 = FrontMatter.extractAndParse(source);
        const r2 = mjmlValidate(source);
        const handler = (frontMatter, content) => new Template({
            content,
            ...frontMatter,
        });
        return future_1.Future.fromResult((0, result_1.resultMap2)(handler, r1, r2)).inner;
    }
}
class MjmlTemplateLoader {
    constructor(source) {
        const parser = new TemplateParser();
        this.loader = new template_1.TemplateLoader(source, parser);
    }
    fetch(key) {
        return this.loader.fetch(key);
    }
}
exports.MjmlTemplateLoader = MjmlTemplateLoader;
class MjmlFileLoader {
    constructor(source) {
        this.loader = new MjmlTemplateLoader(source);
    }
    static filePathTransform(basePath, name) {
        return basePath.join(name + '.mjml');
    }
    static make(fileSystem, path) {
        const source = source_1.FileSystem.make(path, fileSystem, MjmlFileLoader.filePathTransform);
        return future_1.Future.equip(source)
            .map((source) => new MjmlFileLoader(source))
            .inner;
    }
    fetch(key) {
        return this.loader.fetch(key);
    }
}
exports.MjmlFileLoader = MjmlFileLoader;
//# sourceMappingURL=index.js.map