"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileSystem = void 0;
const rustic_1 = require("rustic");
const file_path_1 = require("../internal/file-path");
const future_1 = require("../internal/future");
const defaultTransform = (path, name) => path.join(name);
class FileSystem {
    constructor(basePath, fileSystem, transform) {
        this.basePath = basePath;
        this.fileSystem = fileSystem;
        this.transform = (0, rustic_1.equip)(transform).unwrapOr(defaultTransform);
    }
    static make(path, fileSystem, transform) {
        return future_1.Future.resolve(new file_path_1.FilePath(path))
            .mapErr((e) => JSON.stringify(e))
            .andThen((path) => path.isReadableDirectory(fileSystem))
            .map((path) => new FileSystem(path, fileSystem, transform))
            .inner;
    }
    fetch(key) {
        const fileSystem = this.fileSystem;
        const future = this.transform(this.basePath, key).isReadableFile(this.fileSystem);
        return future_1.Future.equip(future)
            .map((path) => path.toString())
            .andThen((path) => fileSystem.readFile(path)).inner;
    }
}
exports.FileSystem = FileSystem;
//# sourceMappingURL=file-system.js.map